// ============================================================================
// Licensed Materials - Property of IBM
// Project  Zero
//
// (C) Copyright IBM Corp. 2008  All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
// ============================================================================
dojo.provide("zero.webtools.CodeViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dijit.form.Form");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit.layout.ContentPane");

dojo.declare(
    "zero.webtools.CodeViewer",
    [dijit._Widget, dijit._Templated],
	{
		isContainer: true,
		widgetsInTemplate: true,
		
		templateString: "<div class='codeviewer'>" +
			//"<div dojoType='dijit.Dialog' dojoAttachPoint='gotoDialog' duration=10 title='Jump to line'><span>Go to line number:</span><form dojoType='dijit.form.Form' action='javascript:void(0)' dojoAttachPoint='gotoForm' execute='this.submit();'><input dojoType='dijit.form.TextBox' dojoAttachPoint='gotoDialogLineNumber'/><div align='right'><div dojoType='dijit.form.Button' type='submit'>Go</div></div></form></div>"
			"<div dojoType='dijit.layout.LayoutContainer' dojoAttachPoint='mainContainer' style='width: 100%; height: 100%;'>" +
			"<div dojoType='dijit.layout.ContentPane' layoutAlign='top' class='toolbarContainer'><div dojoType='dijit.Toolbar' dojoAttachPoint='editorToolbar'>" +
			//"<div dojoType='dijit.form.Button' dojoAttachPoint='gotoLineButton' dojoAttachEvent='onClick: onPromptGotoLineNumber' iconClass='dijitEditorIcon dijitEditorIconGoto' showLabel='true'>Go to line</div></div>"
			"</div><div dojoType='dijit.layout.ContentPane' dojoattachpoint='editorContentPane' layoutAlign='client' style='border: 1px solid #848484; border-top: 0 none; overflow:hidden;'><div class='lineNumbers' dojoAttachPoint='lineNode' style='float:left;'><div dojoAttachPoint='numbers'>1:</div></div><div class='codeviewerinner' dojoAttachPoint='editorNode' dojoAttachEvent='onscroll: onScroll' style='float:left; overflow:auto;'></div></div></div></div>",
		
		editWidget : null,

        postCreate: function() {        
            this.connect(window, "onresize", "refresh");			
        },

		startup: function() {
            this._titleNode = dojo.doc.createElement("div"); //set title 
            this._titleNode.className = "dijit dijitLeft dijitInline dijitButton dijitButton title";             
            this.editorToolbar.addChild(new dijit._Widget({}, this._titleNode));
			this.refresh();
			this.onOpen();
		},
				
		_data: "",
		lineCount: 1,
		setData: function(data) {
			//dojox.highlight.init(this.contentNode);
			this._data = data;
			this._titleNode.innerHTML=this.fileName;
			this.editorNode.innerHTML = ''+this._escapeXml(this._data)+'';
			this.numbers.innerHTML='';
			this.lineCount = this._countLines(1,this.editorNode.childNodes);
			this.onLinesChanged(1,this.lineCount);
			this.onGotoLine(1);
		},
		_countLines: function(lineNumber, startNodes){
			if (!startNodes) {return lineNumber;}
			for (var i=0; (i < startNodes.length); i++) {
				if ((startNodes[i].nodeType == 1) && (startNodes[i].nodeName == "BR") &&
				    (startNodes[i].nextSibling)) {
					lineNumber = this._countLines(lineNumber+1, startNodes[i].childNodes);
					
				} else {
					lineNumber = this._countLines(lineNumber, startNodes[i].childNodes);									
				}
			}
			return lineNumber;
		},
		onPromptGotoLineNumber: function() {
			var dialog = this.gotoDialog;
			var widget = this;
			var execgoto = dojo.hitch(widget, widget.onGotoLine); 
			this.gotoForm.onExecute = function(){
				var line = 1*widget.gotoDialogLineNumber.getValue();
				if(line <= 0){
					return;
				}
				dialog.hide();
				execgoto(line);
			};
			dialog.show();
			setTimeout(dojo.hitch(this.gotoDialogLineNumber, this.gotoDialogLineNumber.focus),50);			
		},
		onGotoLine: function(lineNumber) {
			var body = this.editorNode;
			body.scrollTop = 13*(lineNumber-1);
		},
		onOpen: function() {
			var bindArgs = {
			    url: this.filePath,
		        load: dojo.hitch(this, function(data, args) { 
					this.onOpenComplete(true, data, args);
				}),
		        error: dojo.hitch(this, function(type, error) { 
					console.debug("Open Error: "+type+"/"+error);	
					this.onOpenComplete(false, type, error);
				}),
			    preventCache: true,
			    handleAs: "text"        
			};
		  	var request = dojo.xhrGet(bindArgs);								
		},		
		onScroll: function() {
			var body = this.editorNode;
			this.numbers.style.top = "-"+body.scrollTop+"px";			
		},
		onOpenComplete: function(success, data, args) {		
			if (success){
			    var contenttype = null;
				var headers = args.xhr.getAllResponseHeaders().split("\n");
				for (var i=0; i < headers.length; i++) {
					var contenttypepos = headers[i].indexOf("Content-Type:")
					if (contenttypepos == 0) {
						contenttype = headers[i].substr(headers[i].indexOf(":")+1);	
					}
				}
				if (/(text\/.*|application\/(x-javascript|javascript|json|xml|ecmascript|html|.*\+xml))/i.test(contenttype)) {
					this.setData(data);
				} else {
					this.setData("Binary data (Content-Type:  "+contenttype+")");
				} 
				this.refresh();
			}
		},			
		onLinesChanged: function(start, end) {
			if (start == 0) start = 1;
			if (start<=end) {
				for (var i=start; i <= end; i++) {
					this.numbers.appendChild(document.createTextNode(i+":"));
					this.numbers.appendChild(document.createElement("br"));
				}
			} 
		},
		_escapeXml: function(/*String*/str, /*Boolean*/noSingleQuotes){
			//summary:
			//		Adds escape sequences for special characters in XML: &<>"
			str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;").replace(/\n/gm, "<br/>").replace(/\t/gm, "    ");
			return str.replace(/\s/gm, "&nbsp;"); // string
		},	
		
		_dirtyView : false,
		refresh: function(w, h) {
			var topContainer = this.domNode.parentNode;
			var box = dojo.contentBox(topContainer);
	        console.debug ("CodeViewer: box.h="+box.h+ " box.w="+box.w);

			var newHeight = box.h-25;
			if(dojo.isIE){
				newHeight = newHeight+3;
			}
			var newWidth = box.w-3;// document.body.clientWidth-5;
			if(dojo.isIE){
				newWidth = newWidth+2;
			}			
			if (box.h==0 || box.w==0) { return; }
			
			this.lineNode.style.height = newHeight+'px';
			this.editorNode.style.height = newHeight+'px';	
			
			var width = 16;
			for (var i=this.lineCount; i >= 10; i=i/10) {
				width = width + 8;
				if(dojo.isIE){
					width = width + 1;
				}
			}
			this.lineNode.style.width = width+'px';
			
			this.editorNode.style.width = (newWidth-width)+'px';							
			this._titleNode.style.width = (newWidth-20)+'px';
			this.mainContainer.resize();
		}	
	}
);