/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * Project  Zero
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
import zero.core.fileserver.FileUtils

boolean alreadyExists(parent, childFile) {
    def currentPath = childFile.getPath();
    def fileName = childFile.getName();
    
    def idx = fileName.lastIndexOf('.')
    def name = (idx > 0) ? fileName.substring(0, idx) : fileName;
	
	if (currentPath.startsWith("app/")) {
		if (currentPath.startsWith("app/resources")) {
			exts = config.resources.defaultExtensions[]
		} else {
			exts = config.fileserver.defaultExtensions[]
		}
		
		if (!currentPath.startsWith("app/scripts") && exts.contains(FileUtils.getFileExtension(fileName))) {
			return existsWithExtension(parent, name, exts)
		} else {
			return existsWithExactName(parent, fileName)
		}
	} else {
		// we are in public, so the only files we need special handling for are index files
		if ("index" == name && config.fileserver.defaultExtensions[].contains(FileUtils.getFileExtension(fileName))) {
			return existsWithExtension(parent, name, config.fileserver.defaultExtensions[])
		} else {
			return existsWithExactName(parent, fileName)
		}
	}
}

boolean existsWithExtension(parent, name,extensions) {
	// give me a name without any extension and what extensions to look for in the results
	return extensions.any { ext ->
		def fullname = parent.path + "/" + name + ext
		parent.files.any { existingChild ->
		  existingChild.path == fullname
		}
	}
}

boolean existsWithExactName(parent, name) {
	def fullname = parent.path + "/" + name
	   // give me an exact name (with extension) to match
	return parent.files.any { existingChild ->
	   existingChild.path == fullname
	}
}

def createFolder(path) {
    int idx = path.lastIndexOf("/")
    def name = path.substring(idx+1)
	// recursive function to populate the files of this parent
	def parent = [ 'type' : 'folder', 'name' : name, 'path' : path, 'files' : [] ]
	config.dependencies[].each { dependency ->
		def dependencyRoot = config.dependencies."${dependency}"[]
		def physicalFile = new File(dependencyRoot, parent.path)
		if (physicalFile.exists()) {
			physicalFile.eachFile { childFile ->
				// ignore duplicate folder/files since only first one "counts"
				// also ignore folders/files starting with .
				if (!(childFile.getName().startsWith('.')) && !(alreadyExists(parent, childFile))) {
			    	parent.files << ['type' : (childFile.isDirectory()) ? 'folder' : 'file', 'path' : parent.path + '/' + childFile.getName()]
				}
			}
		}
	}
    // sort the files by type then name
    parent.files.sort{a,b -> (a.type == b.type) ? a.path.compareTo(b.path) : ((a.type == 'folder') ? -1 : 1) }
    return parent
}

def createItem(path) {
    // Lets find the file or folder
    for (dependency in config.dependencies[]) {
		def dependencyRoot = config.dependencies."${dependency}"[]
		def physicalFile = new File(dependencyRoot, path)
		if (physicalFile.isDirectory()) {
		    return createFolder(path)
		} else if (physicalFile.isFile()){
		    return [ 'type' : 'file', 'name' : physicalFile.getName() + '  (' + dependency +')', 'path' : path ]
		}
	}
    return [ 'type' : 'error', 'name' : 'Error at ' + path, 'path' : path]
}
	
def onGET() {
    def pathInfo = event.pathInfo[]
    def data = null;
    
    // deal with root
    if (pathInfo.length() == 0) {
        data = []
        ['/app', '/public'].each{data << createFolder(it)}
    } else {
        data = createItem(pathInfo)
    }
    
	request.view = 'JSON'
	request.json.output = data
	zero.core.views.ViewEngine.render()
}